#include <sys/param.h>
#include <sys/sysctl.h>
#include <prop/proplib.h>
#include <err.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int
main(int argc, char **argv)
{
	prop_dictionary_t plist = NULL;
	int c;
	bool set_acl = false;
	bool get_acl = false;
	char *filename;

	while ((c = getopt(argc, argv, "f:gsx:")) != -1) {
		switch (c) {
		case 'f':
			filename = optarg;
			break;

		case 'g':
			get_acl = true;
			break;

		case 's':
			set_acl = true;
			break;

		case 'x':
			plist = prop_dictionary_internalize_from_file(optarg);
			if (plist == NULL)
				err(EXIT_FAILURE, "%s", optarg);
			break;

		default:
			break;
		}
	}

	/* Set ACL. */
	if (set_acl && plist != NULL) {
		char *plistxml;
		int r;

		plistxml = prop_dictionary_externalize(plist);
		r = sysctlbyname("security.acl.set", NULL, NULL, plistxml,
		    strlen(plistxml) + 1);
		if (r == -1) {
			err(EXIT_FAILURE, "sysctlbyname");
		}
	}

	/* Get ACL. */
	if (get_acl && filename != NULL) {
		prop_dictionary_t dict;
		char *plistxml, *aclplist;
		size_t len, aclplistlen;
		int r;

		dict = prop_dictionary_create();

		prop_dictionary_set_cstring(dict, "filename", filename);

		plistxml = prop_dictionary_externalize(dict);
		len = strlen(plistxml) + 1;

		aclplistlen = 1024;
		aclplist = calloc(1, aclplistlen);

		r = sysctlbyname("security.acl.get", plistxml, &len, aclplist,
		    aclplistlen);
		if (r == -1) {
			err(EXIT_FAILURE, "sysctlbyname");
		}

		(void)printf("%s\n", aclplist);
	}

	return 0;
}

