/* $NetBSD$ */
/*-
 * Copyright (c) 2009 Elad Efrat <elad@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _ACL_CORE_H_
#define	_ACL_CORE_H_

#include <sys/types.h>
#include <sys/queue.h>

#define	ACE_READ	(1 << 0)
#define	ACE_NO_READ	(1 << 1)
#define	ACE_WRITE	(1 << 2)
#define	ACE_NO_WRITE	(1 << 3)
#define	ACE_EXEC	(1 << 4)
#define	ACE_NO_EXEC	(1 << 5)

#define	ACE_ALLOW_ID	(1 << 6)
#define	ACE_DENY_ID	(1 << 7)
#define	ACE_ALLOW_ALL	(1 << 8)
#define	ACE_DENY_ALL	(1 << 9)

enum ace_id_t { ACE_USER, ACE_GROUP };

/* Access control spec. */
struct ace_spec {
	uint64_t spec_flags;
	LIST_ENTRY(ace_spec) spec_list;
};

struct ace_id {
	enum ace_id_t id_type;
	union {
		uid_t uid;
		gid_t gid;
	} id_data;
#define	id_uid	id_data.uid
#define	id_gid	id_data.gid
};

/* Access control entry. */
struct ace {
	struct ace_id *ace_id;
	LIST_HEAD(ace_specs, ace_spec) ace_specs;
	LIST_ENTRY(ace) ace_list;
};

struct acl {
	LIST_HEAD(acl_aces, ace) acl_aces;
};

struct ace_spec *ace_spec_new(void);
void ace_spec_free(struct ace_spec *);
void ace_spec_setflags(struct ace_spec *, uint64_t);
struct ace_id *ace_id_new(void);
void ace_id_free(struct ace_id *);
void ace_id_set_uid(struct ace_id *, uid_t);
void ace_id_set_gid(struct ace_id *, gid_t);
struct ace *ace_new(void);
void ace_free(struct ace *);
void ace_set_id(struct ace *, struct ace_id *);
void ace_add(struct ace *, struct ace_spec *);
struct acl *acl_new(void);
void acl_free(struct acl *);
int acl_add(struct acl *, struct ace *);
struct ace *acl_lookup_uid(struct acl *, uid_t);
struct ace *acl_lookup_gid(struct acl *, gid_t);
int acl_test_uid(struct acl *, uid_t, uint64_t);
int acl_test_gid(struct acl *, gid_t, uint64_t);

#endif /* !_ACL_CORE_H_ */

