/* $NetBSD$ */
/*-
 * Copyright (c) 2009 Elad Efrat <elad@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/module.h>
#include <sys/sysctl.h>
#include <sys/kauth.h>

#include <secmodel/acl/acl.h>

MODULE(MODULE_CLASS_MISC, acl, NULL);

static struct sysctllog *_acl_kmod_log;

static int
acl_kmod_init(void)
{
	secmodel_acl_start();
	sysctl_security_acl_setup(&_acl_kmod_log);

	return (0);
}

static int
acl_kmod_fini(void)
{
	secmodel_acl_stop();
	sysctl_teardown(&_acl_kmod_log);

	/* XXX synchronize listener exit... */

	return (0);
}

static int
acl_modcmd(modcmd_t cmd, void *arg)
{

	int error = ENOTTY;

	switch (cmd) {
	case MODULE_CMD_INIT:
		error = acl_kmod_init();
		break;

	case MODULE_CMD_FINI:
		error = acl_kmod_fini();
		break;

	case MODULE_CMD_STAT:
		error = 0;
		break;

	default:
		error = ENOTTY;
		break;
	}

	return (error);
}

